@@ -2,12 +2,12 @@ use std::convert::TryFrom;
 use std::num::NonZeroUsize;
 use std::{
     fs,
-    io::{self, BufRead, Cursor, Read},
+    io::{self, BufRead, BufReader, Cursor, Read},
     path::Path,
 };
 
 use calamine::{open_workbook_auto_from_rs, Data, Reader};
-use flate2::read::GzDecoder;
+use flate2::read::MultiGzDecoder;
 use serde_json::{Map, Value};
 
 use crate::config::Config;
@@ -21,10 +21,11 @@ use crate::CliResult;
 enum SupportedFormat {
     Xls,
     NdJSON,
-    JSONArray,
+    Json,
     Text,
     Npy,
     Tar,
+    Md,
 }
 
 impl SupportedFormat {
@@ -32,10 +33,11 @@ impl SupportedFormat {
         Some(match string {
             "xls" | "xlsx" | "xlsb" | "ods" => Self::Xls,
             "jsonl" | "ndjson" => Self::NdJSON,
-            "json" => Self::JSONArray,
+            "json" => Self::Json,
             "txt" | "text" | "lines" => Self::Text,
             "npy" => Self::Npy,
             "tar" | "tar.gz" => Self::Tar,
+            "md" | "markdown" => Self::Md,
             _ => return None,
         })
     }
@@ -68,16 +70,14 @@ Usage:
     xan from --help
 
 Supported formats:
-    ods    - OpenOffice spreadsheet
-    xls    - Excel spreasheet
-    xlsb   - Excel spreasheet
-    xlsx   - Excel spreasheet
-    json   - JSON array or object
-    ndjson - Newline-delimited JSON
-    jsonl  - Newline-delimited JSON
-    txt    - text lines
-    npy    - Numpy array
-    tar    - Tarball archive
+    - ods: OpenOffice spreadsheet
+    - xls, xlsb, xlsx: Excel spreadsheet
+    - json: JSON array or object
+    - ndjson, jsonl: newline-delimited JSON data
+    - txt: text lines
+    - npy: numpy array
+    - tar: tarball archive
+    - md, markdown: Markdown table
 
 Some formats can be streamed, some others require the full file to be loaded into
 memory. The streamable formats are `ndjson`, `jsonl`, `tar`, `txt` and `npy`.
@@ -95,7 +95,12 @@ from options:
                            work with.
 
 Excel/OpenOffice-related options:
-    -s, --sheet <name>     Name of the sheet to convert. [default: Sheet1]
+    --sheet-index <i>    0-based index of the sheet to convert. Defaults to converting
+                         the first sheet. Use -s/--sheet alternatively to select a
+                         sheet by name.
+                         [default: 0]
+    --sheet-name <name>  Name of the sheet to convert.
+    --list-sheets        Print sheet names instead of converting file.
 
 JSON options:
     --sample-size <n>      Number of records to sample before emitting headers.
@@ -109,6 +114,11 @@ Text lines options:
     -c, --column <name>    Name of the column to create.
                            [default: value]
 
+Markdown options:
+    -n, --nth-table <n>    Select nth table in document, starting at 0.
+                           Negative index can be used to select from the end.
+                           [default: 0]
+
 Common options:
     -h, --help             Display this message
     -o, --output <file>    Write output to <file> instead of stdout.
@@ -117,13 +127,16 @@ Common options:
 #[derive(Deserialize)]
 struct Args {
     arg_input: Option<String>,
-    flag_sheet: String,
+    flag_sheet_index: usize,
+    flag_sheet_name: Option<String>,
+    flag_list_sheets: bool,
     flag_format: Option<SupportedFormat>,
     flag_output: Option<String>,
     flag_sample_size: NonZeroUsize,
     flag_key_column: String,
     flag_value_column: String,
     flag_column: String,
+    flag_nth_table: isize,
 }
 
 impl Args {
@@ -145,23 +158,45 @@ impl Args {
             }
         });
 
-        let mut wtr = self.writer()?;
-
         let mut workbook = open_workbook_auto_from_rs(reader)?;
+
+        if self.flag_list_sheets {
+            let mut wtr = Config::new(&self.flag_output).io_writer()?;
+
+            for sheet_name in workbook.sheet_names() {
+                writeln!(&mut wtr, "{}", sheet_name)?;
+            }
+
+            return Ok(());
+        }
+
+        let mut wtr = self.writer()?;
         let mut record = csv::StringRecord::new();
 
-        let range = workbook.worksheet_range(&self.flag_sheet);
+        let range = match &self.flag_sheet_name {
+            Some(name) => Some(workbook.worksheet_range(name)),
+            None => workbook.worksheet_range_at(self.flag_sheet_index),
+        };
 
         match range {
-            Err(_) => {
+            None => {
+                let sheets = workbook.sheet_names().len();
+
+                Err(format!(
+                    "--sheet-index {} is out-of-bounds (number of sheets: {})!",
+                    self.flag_sheet_index, sheets
+                ))?;
+            }
+            Some(Err(_)) => {
                 let sheets = workbook.sheet_names().join(", ");
 
-                return Err(CliError::Other(format!(
+                Err(format!(
                     "could not find the \"{}\" sheet\nshould be one of: {}",
-                    &self.flag_sheet, sheets
-                )));
+                    self.flag_sheet_name.as_ref().unwrap(),
+                    sheets
+                ))?;
             }
-            Ok(range) => {
+            Some(Ok(range)) => {
                 for row in range.rows() {
                     record.clear();
 
@@ -191,7 +226,7 @@ impl Args {
 
     fn convert_ndjson(&self) -> CliResult<()> {
         let mut wtr = self.writer()?;
-        let rdr = Config::new(&self.arg_input).io_buf_reader()?;
+        let rdr = BufReader::new(Config::new(&self.arg_input).io_reader()?);
 
         for_each_json_value_as_csv_record(
             rdr.lines().map(|line| -> Result<Value, CliError> {
@@ -207,7 +242,7 @@ impl Args {
         Ok(wtr.flush()?)
     }
 
-    fn convert_json_array(&self) -> CliResult<()> {
+    fn convert_json(&self) -> CliResult<()> {
         let mut rdr = Config::new(&self.arg_input).io_reader()?;
 
         let mut contents = String::new();
@@ -251,7 +286,7 @@ impl Args {
     }
 
     fn convert_text_lines(&self) -> CliResult<()> {
-        let rdr = Config::new(&self.arg_input).io_buf_reader()?;
+        let rdr = BufReader::new(Config::new(&self.arg_input).io_reader()?);
         let mut wtr = self.writer()?;
         wtr.write_record([&self.flag_column])?;
 
@@ -346,7 +381,7 @@ impl Args {
             Some(p) => Box::new(fs::File::open(p)?),
         };
 
-        let rdr = GzDecoder::new(rdr);
+        let rdr = MultiGzDecoder::new(rdr);
         let mut archive = tar::Archive::new(rdr);
 
         let mut wtr = self.writer()?;
@@ -370,7 +405,7 @@ impl Args {
             bytes.clear();
 
             if entry.path_bytes().ends_with(b".gz") {
-                let mut inner_gz = GzDecoder::new(&mut entry);
+                let mut inner_gz = MultiGzDecoder::new(&mut entry);
                 inner_gz.read_to_end(&mut bytes)?;
             } else {
                 entry.read_to_end(&mut bytes)?;
@@ -386,6 +421,89 @@ impl Args {
 
         Ok(wtr.flush()?)
     }
+
+    fn convert_markdown(&self) -> CliResult<()> {
+        use comrak::nodes::NodeValue;
+        use comrak::{parse_document, Arena, Options};
+
+        let mut rdr = Config::new(&self.arg_input).io_reader()?;
+        let mut buf = String::new();
+        rdr.read_to_string(&mut buf)?;
+
+        let arena = Arena::new();
+        let mut options = Options::default();
+        options.extension.table = true;
+        let root = parse_document(&arena, &buf, &options);
+        let tables = root
+            .descendants()
+            .filter(|n| matches!(n.data.borrow().value, NodeValue::Table(_)))
+            .collect::<Vec<_>>();
+
+        if tables.is_empty() {
+            Err("target Markdown does not contain a table")?;
+        }
+        let table = usize::try_from(self.flag_nth_table)
+            .ok()
+            // select from end if negative.
+            .or_else(|| tables.len().checked_add_signed(self.flag_nth_table))
+            .and_then(|i| tables.get(i))
+            .ok_or_else(|| {
+                let bounds = if self.flag_nth_table >= 0 {
+                    [0, tables.len()].map(|n| n.to_string())
+                } else {
+                    // Saturating to avoid underflow.
+                    // isize::MIN is smallest supported number anyway due to type of `flag_select`.
+                    let low = 0isize.saturating_sub_unsigned(tables.len());
+                    [-1, low].map(|n| n.to_string())
+                };
+                format!(
+                    "table index {} is out of bounds in target Markdown (must be between {} and {})",
+                    self.flag_nth_table,
+                    bounds[0],
+                    bounds[1]
+                )
+            })?;
+
+        let mut wtr = self.writer()?;
+        let mut record = csv::ByteRecord::new();
+        let lines = buf.lines().collect::<Vec<_>>();
+        for row in table.children() {
+            // Ignore non-row nodes, though there shouldn't be any.
+            if !matches!(row.data.borrow().value, NodeValue::TableRow(_)) {
+                continue;
+            }
+            for cell in row.children() {
+                // Ignore non-cell nodes, though there shouldn't be any.
+                if !matches!(cell.data.borrow().value, NodeValue::TableCell) {
+                    continue;
+                }
+
+                // `cell.to_string()` drops formatting so extract raw string from `buf`.
+                // Position of whole cell includes padding so get range from start
+                // of first child to end of last child.
+                if let Some((start, end)) = (|| {
+                    let first = cell.first_child()?.data.borrow();
+                    let last = cell.last_child()?.data.borrow();
+                    Some((first.sourcepos.start, last.sourcepos.end))
+                })() {
+                    if start.line != end.line {
+                        // Markdown does not support multiline table cells
+                        // so this shouldn't happen.
+                        return Err("Unsupported multiline Markdown table cell".into());
+                    }
+                    // sourcepos is 1-based, inclusive, and by bytes not characters
+                    let line = lines[start.line - 1].as_bytes();
+                    record.push_field(&line[start.column - 1..end.column]);
+                } else {
+                    record.push_field(&[]);
+                }
+            }
+            wtr.write_byte_record(&record)?;
+            record.clear();
+        }
+
+        Ok(wtr.flush()?)
+    }
 }
 
 pub fn run(argv: &[&str]) -> CliResult<()> {
@@ -413,9 +531,10 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
     match target_format {
         SupportedFormat::Xls => args.convert_xls(),
         SupportedFormat::NdJSON => args.convert_ndjson(),
-        SupportedFormat::JSONArray => args.convert_json_array(),
+        SupportedFormat::Json => args.convert_json(),
         SupportedFormat::Text => args.convert_text_lines(),
         SupportedFormat::Npy => args.convert_npy(),
         SupportedFormat::Tar => args.convert_tar(),
+        SupportedFormat::Md => args.convert_markdown(),
     }
 }
