@@ -18,29 +18,28 @@ pub mod fixlengths;
 pub mod flatmap;
 pub mod flatten;
 pub mod fmt;
-pub mod foreach;
 pub mod frequency;
 pub mod from;
+pub mod fuzzy_join;
 pub mod groupby;
+pub mod head;
 pub mod headers;
 pub mod heatmap;
 pub mod help;
 pub mod hist;
 pub mod implode;
-pub mod index;
 pub mod input;
 pub mod join;
 pub mod map;
 pub mod matrix;
 pub mod merge;
-mod moonblade;
 pub mod network;
 pub mod parallel;
 pub mod partition;
+pub mod pivot;
 pub mod plot;
 pub mod progress;
 pub mod range;
-pub mod regex_join;
 pub mod rename;
 pub mod reverse;
 pub mod sample;
@@ -52,11 +51,13 @@ pub mod slice;
 pub mod sort;
 pub mod split;
 pub mod stats;
+pub mod tail;
 pub mod to;
 pub mod tokenize;
 pub mod top;
 pub mod transform;
 pub mod transpose;
-pub mod url_join;
+pub mod unpivot;
 pub mod view;
 pub mod vocab;
+pub mod window;
