@@ -2,6 +2,7 @@
 //  - https://github.com/ratatui/ratatui/issues/334
 //  - https://github.com/ratatui/ratatui/issues/1391
 use std::convert::TryFrom;
+use std::io::{stderr, stdout, Write};
 use std::num::NonZeroUsize;
 
 use ahash::RandomState;
@@ -93,7 +94,7 @@ plot options:
     -L, --line                 Whether to draw a line plot instead of the default scatter plot.
     -B, --bars                 Whether to draw bars instead of the default scatter plot.
                                WARNING: currently does not work if y range does not include 0.
-                               (https://github.com/ratatui/ratatui/issues/1391)
+                               https://github.com/ratatui/ratatui/issues/1391
     -T, --time                 Use to indicate that the x axis is temporal. The axis will be
                                discretized according to some inferred temporal granularity and
                                y values will be summed wrt the newly discretized x axis.
@@ -472,7 +473,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
     }
 
     if series_builder.is_empty() {
-        println!("Nothing to display!");
+        writeln!(&mut stderr(), "Nothing to display!")?;
         return Ok(());
     }
 
@@ -660,7 +661,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
                 if first_grid_col {
                     first_grid_col = false;
                 } else {
-                    println!();
+                    writeln!(&mut stdout())?;
                 }
 
                 print_ratatui_frame_to_stdout(cols, rows, |frame| {
