@@ -4,7 +4,7 @@ use std::path::PathBuf;
 use std::time::Duration;
 
 use bytesize::MB;
-use flate2::read::GzDecoder;
+use flate2::read::MultiGzDecoder;
 use indicatif::{HumanCount, ProgressBar, ProgressStyle};
 
 use crate::config::{Config, Delimiter};
@@ -175,8 +175,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
     console::set_colors_enabled(true);
 
     if args.flag_bytes {
-        let (total, file, is_gzipped): (Option<u64>, Box<dyn io::Read>, bool) = match args.arg_input
-        {
+        let (total, file, is_gzipped): (Option<u64>, Box<dyn Read>, bool) = match args.arg_input {
             None => (None, Box::new(io::stdin()), false),
             Some(p) => {
                 let is_gzipped = p.ends_with(".gz");
@@ -195,7 +194,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         let mut wrapper: Box<dyn Read> = Box::new(bar.inner.wrap_read(file));
 
         if is_gzipped {
-            wrapper = Box::new(GzDecoder::new(wrapper));
+            wrapper = Box::new(MultiGzDecoder::new(wrapper));
         }
 
         let mut wtr = Config::new(&args.flag_output).io_writer()?;
