@@ -198,7 +198,7 @@ the -e/--expand and -C/--force-colors flags before piping like so:
 Finally, it is possible to customize the default behavior of this command through
 the \"XAN_VIEW_ARGS\" environment variable. This variable takes a series of
 supported flags: -t/--theme, -p/--pager, -l/--limit, -R/--rainbow, -E/--sanitize-emojis,
-and -S/--significance, -I/--hide-index.
+and -S/--significance, -I/--hide-index & -M/--hide-info.
 
 So if you want, for instance, to use the borderles theme, hide the index column and
 restrict the number of floating points decimals to be shown by default:
@@ -238,6 +238,7 @@ view options:
     -M, --hide-info         Hide information about number of displayed columns, rows etc.
     -g, --groupby <cols>    Isolate and emphasize groups of rows, represented by consecutive
                             rows with identical values in selected columns.
+    -r, --right <col>       Force right alignment of selected columns.
 
 Common options:
     -h, --help             Display this message
@@ -266,6 +267,7 @@ struct Args {
     flag_hide_headers: bool,
     flag_hide_info: bool,
     flag_groupby: Option<SelectColumns>,
+    flag_right: Option<SelectColumns>,
     flag_significance: Option<NonZeroUsize>,
 }
 
@@ -297,6 +299,10 @@ impl Args {
             from_argv.flag_hide_index = true;
         }
 
+        if !from_argv.flag_hide_info && from_env.flag_hide_info {
+            from_argv.flag_hide_info = true;
+        }
+
         if !from_argv.flag_pager && from_env.flag_pager {
             from_argv.flag_pager = true;
         }
@@ -361,13 +367,29 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         .select(args.flag_select.clone());
 
     let mut rdr = rconfig.reader()?;
-    let byte_headers = rdr.byte_headers()?;
-    let sel = rconfig.selection(byte_headers)?;
+    let byte_headers = rdr.byte_headers()?.clone();
+    let mut sel = rconfig.selection(&byte_headers)?;
 
+    // NOTE: the groupby selection logic is a bit complex because it must work
+    // in conjunction with --select correctly.
     let mut groupby_sel_opt = args
         .flag_groupby
         .clone()
-        .map(|cols| cols.selection(byte_headers, !args.flag_no_headers))
+        .map(|cols| cols.selection(&byte_headers, !args.flag_no_headers))
+        .transpose()?;
+
+    if let Some(groupby_sel) = &groupby_sel_opt {
+        for i in groupby_sel.iter().rev() {
+            if !sel.contains(*i) {
+                sel.insert(0, *i);
+            }
+        }
+    }
+
+    groupby_sel_opt = args
+        .flag_groupby
+        .clone()
+        .map(|cols| cols.selection(&sel.select(&byte_headers).collect(), !args.flag_no_headers))
         .transpose()?;
 
     if let (Some(groupby_sel), false) = (&mut groupby_sel_opt, args.flag_hide_index) {
@@ -375,12 +397,20 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
     }
 
     let headers = rdr.headers()?.clone();
-    let mut headers = sel
-        .select_string_record(&headers)
-        .collect::<csv::StringRecord>();
+    let mut headers = sel.select(&headers).collect::<csv::StringRecord>();
+
+    let mut right_sel_opt = args
+        .flag_right
+        .as_ref()
+        .map(|s| s.selection(headers.as_byte_record(), !args.flag_no_headers))
+        .transpose()?;
 
     if !args.flag_hide_index {
         headers = headers.prepend(theme.index_column_header);
+
+        if let Some(right_sel) = &mut right_sel_opt {
+            right_sel.offset_by(1);
+        }
     }
 
     if rconfig.no_headers {
@@ -404,7 +434,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
     let mut number_formatter = args.flag_significance.map(|s| {
         Formatter::new()
             .precision(Precision::Significance(s.get() as u8))
-            .separator(',')
+            .separator(None)
             .unwrap()
     });
 
@@ -420,7 +450,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
                 None => break,
                 Some((i, record)) => {
                     let mut record = sel
-                        .select_string_record(&record?)
+                        .select(&record?)
                         .map(|cell| {
                             let mut cell = cell.to_string();
 
@@ -460,6 +490,10 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         records
     };
 
+    if records.is_empty() && byte_headers.is_empty() {
+        Err("either input is completely empty or piped process errored upstream!")?;
+    }
+
     let need_to_repeat_headers = match rows {
         None => true,
         Some(r) => records.len() + HEADERS_ROWS > r,
@@ -483,6 +517,30 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         })
         .collect();
 
+    // Alignment inferrence
+    // TODO: type tagging for values should happen on initial read
+    let right_sel_mask_opt = right_sel_opt.map(|sel| sel.mask(headers.len()));
+
+    let alignments = (0..headers.len())
+        .map(|i| {
+            if !args.flag_hide_index && i == 0 {
+                return false;
+            }
+
+            if let Some(right_sel_mask) = &right_sel_mask_opt {
+                if right_sel_mask[i] {
+                    return true;
+                }
+            }
+
+            records.iter().all(|r| {
+                let cell = &r[i];
+
+                cell.is_empty() || cell.parse::<i64>().is_ok()
+            })
+        })
+        .collect::<Vec<_>>();
+
     // Width inferrence
     let displayed_columns = infer_best_column_display(
         cols,
@@ -719,7 +777,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
     for record in records.iter() {
         let (need_to_draw_hr, need_to_erase_sel) = if let Some(groupby_sel) = &groupby_sel_opt {
             let current_key = groupby_sel
-                .select_string_record(record)
+                .select(record)
                 .map(|cell| cell.to_string())
                 .collect::<Vec<_>>();
 
@@ -770,7 +828,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
                     util::colorize(
                         &colorizer,
                         &util::unicode_aware_highlighted_pad_with_ellipsis(
-                            false,
+                            alignments[col.index],
                             cell,
                             col.allowed_width,
                             " ",
@@ -788,7 +846,15 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
     if !all_records_buffered {
         let row: Vec<colored::ColoredString> = displayed_columns
             .iter()
-            .map(|col| util::unicode_aware_rpad_with_ellipsis("…", col.allowed_width, " ").dimmed())
+            .map(|col| {
+                util::unicode_aware_pad_with_ellipsis(
+                    alignments[col.index],
+                    "…",
+                    col.allowed_width,
+                    " ",
+                )
+                .dimmed()
+            })
             .collect();
 
         write_row(row, record_i % 2 == 0)?;
