@@ -1,20 +1,24 @@
+use std::collections::{BTreeMap, BTreeSet};
+use std::iter::once;
 use std::sync::Arc;
 
+use ahash::RandomState;
 use csv::ByteRecord;
+use indexmap::IndexMap;
 use jiff::{civil::DateTime, Unit};
 
 use super::aggregators::{
     AllAny, ApproxCardinality, ApproxQuantiles, ArgExtent, ArgTop, Count, CovarianceWelford, First,
-    Frequencies, Last, LexicographicExtent, MedianType, Numbers, NumericExtent, Sum, Types, Values,
-    Welford, ZonedExtent,
+    Frequencies, Last, LexicographicExtent, MedianType, Numbers, NumericExtent, RMSWelford, Sum,
+    Types, Values, Welford, ZonedExtent,
 };
 use crate::collections::ClusteredInsertHashmap;
 use crate::moonblade::error::{ConcretizationError, EvaluationError, SpecifiedEvaluationError};
 use crate::moonblade::interpreter::{
-    concretize_expression, eval_expression, ConcreteExpr, EvaluationContext,
+    concretize_expression, eval_expression_with_optional_last_value, ConcreteExpr,
 };
 use crate::moonblade::parser::{parse_aggregations, Aggregations};
-use crate::moonblade::types::{DynamicNumber, DynamicValue, FunctionArguments};
+use crate::moonblade::types::{DynamicNumber, DynamicValue, FunctionArguments, HeadersIndex};
 
 // NOTE: we are boxing some ones to avoid going over size=64
 #[derive(Debug, Clone)]
@@ -22,7 +26,7 @@ enum Aggregator {
     AllAny(AllAny),
     ApproxCardinality(Box<ApproxCardinality>),
     ApproxQuantiles(Box<ApproxQuantiles>),
-    ArgExtent(ArgExtent),
+    ArgExtent(Box<ArgExtent>),
     ArgTop(ArgTop),
     Count(Count),
     CovarianceWelford(CovarianceWelford),
@@ -31,8 +35,9 @@ enum Aggregator {
     Last(Last),
     Values(Values),
     LexicographicExtent(LexicographicExtent),
-    Frequencies(Frequencies),
+    Frequencies(Box<Frequencies>),
     Numbers(Numbers),
+    RMSWelford(RMSWelford),
     Sum(Sum),
     Types(Types),
     Welford(Welford),
@@ -58,6 +63,7 @@ impl Aggregator {
             LexicographicExtent(inner) => inner.clear(),
             Frequencies(inner) => inner.clear(),
             Numbers(inner) => inner.clear(),
+            RMSWelford(inner) => inner.clear(),
             Sum(inner) => inner.clear(),
             Types(inner) => inner.clear(),
             Welford(inner) => inner.clear(),
@@ -72,7 +78,7 @@ impl Aggregator {
             (AllAny(inner), AllAny(other_inner)) => inner.merge(other_inner),
             (ApproxCardinality(inner), ApproxCardinality(other_inner)) => inner.merge(*other_inner),
             (ApproxQuantiles(inner), ApproxQuantiles(other_inner)) => inner.merge(*other_inner),
-            (ArgExtent(inner), ArgExtent(other_inner)) => inner.merge(other_inner),
+            (ArgExtent(inner), ArgExtent(other_inner)) => inner.merge(*other_inner),
             (ArgTop(inner), ArgTop(other_inner)) => inner.merge(other_inner),
             (Count(inner), Count(other_inner)) => inner.merge(other_inner),
             (CovarianceWelford(inner), CovarianceWelford(other_inner)) => inner.merge(other_inner),
@@ -83,8 +89,9 @@ impl Aggregator {
             (LexicographicExtent(inner), LexicographicExtent(other_inner)) => {
                 inner.merge(other_inner)
             }
-            (Frequencies(inner), Frequencies(other_inner)) => inner.merge(other_inner),
+            (Frequencies(inner), Frequencies(other_inner)) => inner.merge(*other_inner),
             (Numbers(inner), Numbers(other_inner)) => inner.merge(other_inner),
+            (RMSWelford(inner), RMSWelford(other_inner)) => inner.merge(other_inner),
             (Sum(inner), Sum(other_inner)) => inner.merge(other_inner),
             (Types(inner), Types(other_inner)) => inner.merge(other_inner),
             (Welford(inner), Welford(other_inner)) => inner.merge(other_inner),
@@ -111,8 +118,10 @@ impl Aggregator {
     fn get_final_value(
         &self,
         method: &ConcreteAggregationMethod,
-        context: &EvaluationContext,
+        headers_index: &HeadersIndex,
     ) -> Result<DynamicValue, SpecifiedEvaluationError> {
+        debug_assert!(std::mem::size_of::<Self>() <= 64);
+
         Ok(match (method, self) {
             (ConcreteAggregationMethod::All, Self::AllAny(inner)) => {
                 DynamicValue::from(inner.all())
@@ -136,8 +145,14 @@ impl Aggregator {
                     Some(expr) => {
                         let mut strings = Vec::new();
 
-                        for (index, record) in inner.top_records() {
-                            let value = eval_expression(expr, Some(index), &record, context)?;
+                        for (index, record, last_value) in inner.top_records() {
+                            let value = eval_expression_with_optional_last_value(
+                                expr,
+                                Some(index),
+                                &record,
+                                headers_index,
+                                last_value,
+                            )?;
 
                             strings.push(
                                 value
@@ -166,11 +181,12 @@ impl Aggregator {
             (ConcreteAggregationMethod::CovarianceSample, Self::CovarianceWelford(inner)) => {
                 DynamicValue::from(inner.sample_covariance())
             }
-            (ConcreteAggregationMethod::Ratio, Self::Count(inner)) => {
-                DynamicValue::from(inner.ratio())
-            }
-            (ConcreteAggregationMethod::Percentage, Self::Count(inner)) => {
-                DynamicValue::from(inner.percentage())
+            (ConcreteAggregationMethod::Ratio(decimals), Self::Count(inner)) => match decimals {
+                None => DynamicValue::from(inner.ratio()),
+                Some(p) => DynamicValue::from(format!("{:.p$}", inner.ratio(), p = p)),
+            },
+            (ConcreteAggregationMethod::Percentage(decimals), Self::Count(inner)) => {
+                DynamicValue::from(inner.percentage(*decimals))
             }
             (ConcreteAggregationMethod::DistinctValues(separator), Self::Frequencies(inner)) => {
                 DynamicValue::from(inner.join(separator))
@@ -203,10 +219,18 @@ impl Aggregator {
                 DynamicValue::from(inner.min())
             }
             (ConcreteAggregationMethod::ArgMin(expr_opt), Self::ArgExtent(inner)) => {
-                if let Some((index, record)) = inner.argmin() {
+                if let Some((index, record, last_value)) = inner.argmin() {
                     match expr_opt {
                         None => DynamicValue::from(*index),
-                        Some(expr) => return eval_expression(expr, Some(*index), record, context),
+                        Some(expr) => {
+                            return eval_expression_with_optional_last_value(
+                                expr,
+                                Some(*index),
+                                record,
+                                headers_index,
+                                last_value.clone(),
+                            )
+                        }
                     }
                 } else {
                     DynamicValue::None
@@ -231,10 +255,18 @@ impl Aggregator {
                 DynamicValue::from(inner.max())
             }
             (ConcreteAggregationMethod::ArgMax(expr_opt), Self::ArgExtent(inner)) => {
-                if let Some((index, record)) = inner.argmax() {
+                if let Some((index, record, last_value)) = inner.argmax() {
                     match expr_opt {
                         None => DynamicValue::from(*index),
-                        Some(expr) => return eval_expression(expr, Some(*index), record, context),
+                        Some(expr) => {
+                            return eval_expression_with_optional_last_value(
+                                expr,
+                                Some(*index),
+                                record,
+                                headers_index,
+                                last_value.clone(),
+                            )
+                        }
                     }
                 } else {
                     DynamicValue::None
@@ -261,8 +293,8 @@ impl Aggregator {
                 ConcreteAggregationMethod::MostCommonValues(k, separator),
                 Self::Frequencies(inner),
             ) => DynamicValue::from(inner.most_common(*k).join(separator)),
-            (ConcreteAggregationMethod::Sparkline(bins), Self::Numbers(inner)) => {
-                DynamicValue::from(inner.sparkline(*bins))
+            (ConcreteAggregationMethod::Rms, Self::RMSWelford(inner)) => {
+                DynamicValue::from(inner.rms())
             }
             (ConcreteAggregationMethod::Sum, Self::Sum(inner)) => DynamicValue::from(inner.get()),
             (ConcreteAggregationMethod::VarPop, Self::Welford(inner)) => {
@@ -399,8 +431,8 @@ impl CompositeAggregator {
                 upsert_boxed_aggregator!(ApproxQuantiles)
             }
             ConcreteAggregationMethod::Count
-            | ConcreteAggregationMethod::Ratio
-            | ConcreteAggregationMethod::Percentage => {
+            | ConcreteAggregationMethod::Ratio(_)
+            | ConcreteAggregationMethod::Percentage(_) => {
                 upsert_aggregator!(Count)
             }
             ConcreteAggregationMethod::CovariancePop
@@ -426,9 +458,9 @@ impl CompositeAggregator {
                     .iter()
                     .position(|item| matches!(item, Aggregator::NumericExtent(_)))
                 {
-                    None => upsert_aggregator!(ArgExtent),
+                    None => upsert_boxed_aggregator!(ArgExtent),
                     Some(idx) => {
-                        self.methods[idx] = Aggregator::ArgExtent(ArgExtent::new());
+                        self.methods[idx] = Aggregator::ArgExtent(Box::new(ArgExtent::new()));
                         idx
                     }
                 }
@@ -461,8 +493,7 @@ impl CompositeAggregator {
             }
             ConcreteAggregationMethod::Median(_)
             | ConcreteAggregationMethod::Quantile(_)
-            | ConcreteAggregationMethod::Quartile(_)
-            | ConcreteAggregationMethod::Sparkline(_) => {
+            | ConcreteAggregationMethod::Quartile(_) => {
                 upsert_aggregator!(Numbers)
             }
             ConcreteAggregationMethod::Mode
@@ -471,7 +502,10 @@ impl CompositeAggregator {
             | ConcreteAggregationMethod::DistinctValues(_)
             | ConcreteAggregationMethod::MostCommonCounts(_, _)
             | ConcreteAggregationMethod::MostCommonValues(_, _) => {
-                upsert_aggregator!(Frequencies)
+                upsert_boxed_aggregator!(Frequencies)
+            }
+            ConcreteAggregationMethod::Rms => {
+                upsert_aggregator!(RMSWelford)
             }
             ConcreteAggregationMethod::Sum => {
                 upsert_aggregator!(Sum)
@@ -525,12 +559,12 @@ impl CompositeAggregator {
                     }
                     Aggregator::ArgExtent(extent) => {
                         if !value.is_nullish() {
-                            extent.add(index, value.try_as_number()?, record);
+                            extent.add(index, value.try_as_number()?, record, &None);
                         }
                     }
                     Aggregator::ArgTop(top) => {
                         if !value.is_nullish() {
-                            top.add(index, value.try_as_number()?, record);
+                            top.add(index, value.try_as_number()?, record, &None);
                         }
                     }
                     Aggregator::First(first) => {
@@ -563,6 +597,11 @@ impl CompositeAggregator {
                             numbers.add(value.try_as_number()?);
                         }
                     }
+                    Aggregator::RMSWelford(inner) => {
+                        if !value.is_nullish() {
+                            inner.add(value.try_as_f64()?);
+                        }
+                    }
                     Aggregator::Sum(sum) => {
                         if !value.is_nullish() {
                             sum.add(value.try_as_number()?);
@@ -643,9 +682,9 @@ impl CompositeAggregator {
         &self,
         handle: usize,
         method: &ConcreteAggregationMethod,
-        context: &EvaluationContext,
+        headers_index: &HeadersIndex,
     ) -> Result<DynamicValue, SpecifiedEvaluationError> {
-        self.methods[handle].get_final_value(method, context)
+        self.methods[handle].get_final_value(method, headers_index)
     }
 }
 
@@ -706,14 +745,14 @@ fn get_function_arguments_parser(name: &str) -> Option<(FunctionArguments, Argum
             ))
         }),
         "cardinality" => (FunctionArguments::unary(), |_| Ok(Cardinality)),
-        "correlation" => (FunctionArguments::unary(), |_| Ok(Correlation)),
-        "count" => (FunctionArguments::unary(), |_| Ok(Count)),
+        "correlation" => (FunctionArguments::binary(), |_| Ok(Correlation)),
+        "count" => (FunctionArguments::with_range(0..=1), |_| Ok(Count)),
         "count_seconds" => (FunctionArguments::unary(), |_| Ok(CountTime(Unit::Second))),
         "count_hours" => (FunctionArguments::unary(), |_| Ok(CountTime(Unit::Hour))),
         "count_days" => (FunctionArguments::unary(), |_| Ok(CountTime(Unit::Day))),
         "count_years" => (FunctionArguments::unary(), |_| Ok(CountTime(Unit::Year))),
-        "covariance" | "covariance_pop" => (FunctionArguments::unary(), |_| Ok(CovariancePop)),
-        "covariance_sample" => (FunctionArguments::unary(), |_| Ok(CovarianceSample)),
+        "covariance" | "covariance_pop" => (FunctionArguments::binary(), |_| Ok(CovariancePop)),
+        "covariance_sample" => (FunctionArguments::binary(), |_| Ok(CovarianceSample)),
         "distinct_values" => (FunctionArguments::with_range(1..=2), |args| {
             Ok(DistinctValues(cast_as_separator(args.first())?))
         }),
@@ -747,7 +786,17 @@ fn get_function_arguments_parser(name: &str) -> Option<(FunctionArguments, Argum
                 cast_as_separator(args.get(1))?,
             ))
         }),
-        "percentage" => (FunctionArguments::unary(), |_| Ok(Percentage)),
+        "percentage" => (FunctionArguments::with_range(1..=2), |args| {
+            let decimals = if !args.is_empty() {
+                let value =
+                    cast_as_static_value(args.first().unwrap(), DynamicValue::try_as_usize)?;
+                Some(value.min(16))
+            } else {
+                None
+            };
+
+            Ok(Percentage(decimals))
+        }),
         "quantile" => (FunctionArguments::binary(), |args| {
             Ok(Quantile(cast_as_static_value(
                 args.first().unwrap(),
@@ -762,13 +811,18 @@ fn get_function_arguments_parser(name: &str) -> Option<(FunctionArguments, Argum
         }),
         "var" | "var_pop" => (FunctionArguments::unary(), |_| Ok(VarPop)),
         "var_sample" => (FunctionArguments::unary(), |_| Ok(VarSample)),
-        "ratio" => (FunctionArguments::unary(), |_| Ok(Ratio)),
-        "sparkline" => (FunctionArguments::with_range(1..=2), |args| {
-            Ok(Sparkline(match args.first() {
-                Some(arg) => cast_as_static_value(arg, DynamicValue::try_as_usize)?,
-                None => 10,
-            }))
+        "ratio" => (FunctionArguments::with_range(1..=2), |args| {
+            let decimals = if !args.is_empty() {
+                let value =
+                    cast_as_static_value(args.first().unwrap(), DynamicValue::try_as_usize)?;
+                Some(value.min(16))
+            } else {
+                None
+            };
+
+            Ok(Ratio(decimals))
         }),
+        "rms" => (FunctionArguments::unary(), |_| Ok(Rms)),
         "stddev" | "stddev_pop" => (FunctionArguments::unary(), |_| Ok(StddevPop)),
         "stddev_sample" => (FunctionArguments::unary(), |_| Ok(StddevSample)),
         "sum" => (FunctionArguments::unary(), |_| Ok(Sum)),
@@ -814,11 +868,11 @@ enum ConcreteAggregationMethod {
     Modes(String),
     MostCommonValues(usize, String),
     MostCommonCounts(usize, String),
-    Percentage,
+    Percentage(Option<usize>),
     Quartile(usize),
     Quantile(f64),
-    Ratio,
-    Sparkline(usize),
+    Ratio(Option<usize>),
+    Rms,
     Sum,
     Values(String),
     VarPop,
@@ -831,12 +885,16 @@ enum ConcreteAggregationMethod {
 }
 
 impl ConcreteAggregationMethod {
-    fn parse(name: &str, args: &[ConcreteExpr]) -> Result<Self, ConcretizationError> {
+    fn parse(
+        name: &str,
+        args_count: usize,
+        args: &[ConcreteExpr],
+    ) -> Result<Self, ConcretizationError> {
         match get_function_arguments_parser(name) {
             None => Err(ConcretizationError::UnknownFunction(name.to_string())),
             Some((function_arguments, parser)) => {
                 function_arguments
-                    .validate_arity(args.len() + 1)
+                    .validate_arity(args_count)
                     .map_err(|invalid_arity| {
                         ConcretizationError::InvalidArity(name.to_string(), invalid_arity)
                     })?;
@@ -870,6 +928,8 @@ fn concretize_aggregations(
     let mut concrete_aggregations = ConcreteAggregations::new();
 
     for mut aggregation in aggregations {
+        let args_count = aggregation.args.len();
+
         if ["most_common", "most_common_counts", "top", "argtop"]
             .contains(&aggregation.func_name.as_str())
         {
@@ -882,8 +942,6 @@ fn concretize_aggregations(
             .map(|arg| concretize_expression(arg.clone(), headers, None))
             .transpose()?;
 
-        let mut skip: usize = 1;
-
         let pair_expr = if aggregation.args.len() > 1
             && [
                 "covariance",
@@ -893,9 +951,8 @@ fn concretize_aggregations(
             ]
             .contains(&aggregation.func_name.as_str())
         {
-            skip = 2;
             Some(concretize_expression(
-                aggregation.args.get(1).unwrap().clone(),
+                aggregation.args.pop().unwrap().clone(),
                 headers,
                 None,
             )?)
@@ -905,11 +962,11 @@ fn concretize_aggregations(
 
         let mut args: Vec<ConcreteExpr> = Vec::new();
 
-        for arg in aggregation.args.into_iter().skip(skip) {
+        for arg in aggregation.args.into_iter().skip(1) {
             args.push(concretize_expression(arg, headers, None)?);
         }
 
-        let method = ConcreteAggregationMethod::parse(&aggregation.func_name, &args)?;
+        let method = ConcreteAggregationMethod::parse(&aggregation.func_name, args_count, &args)?;
 
         let concrete_aggregation = ConcreteAggregation {
             agg_name: aggregation.agg_name,
@@ -925,8 +982,7 @@ fn concretize_aggregations(
 }
 
 fn prepare(code: &str, headers: &ByteRecord) -> Result<ConcreteAggregations, ConcretizationError> {
-    let parsed_aggregations =
-        parse_aggregations(code).map_err(|_| ConcretizationError::ParseError(code.to_string()))?;
+    let parsed_aggregations = parse_aggregations(code).map_err(ConcretizationError::ParseError)?;
 
     concretize_aggregations(parsed_aggregations, headers)
 }
@@ -945,6 +1001,16 @@ impl PlannerExecutionUnit {
     fn key(&self) -> (&Option<ConcreteExpr>, &Option<ConcreteExpr>) {
         (&self.expr, &self.pair_expr)
     }
+
+    fn used_column_indices(&self, scratch: &mut Vec<usize>) {
+        if let Some(expr) = &self.expr {
+            expr.used_column_indices(scratch);
+        }
+
+        if let Some(expr) = &self.pair_expr {
+            expr.used_column_indices(scratch);
+        }
+    }
 }
 
 // NOTE: output unit are aligned with the list of concrete aggregations and
@@ -1019,6 +1085,12 @@ impl ConcreteAggregationPlanner {
             .collect()
     }
 
+    fn instantiate_single_aggregator(&self) -> CompositeAggregator {
+        debug_assert!(self.execution_plan.len() == 1);
+
+        self.execution_plan[0].aggregator_blueprint.clone()
+    }
+
     fn headers(&self) -> impl Iterator<Item = &[u8]> {
         self.output_plan.iter().map(|unit| unit.agg_name.as_bytes())
     }
@@ -1026,13 +1098,13 @@ impl ConcreteAggregationPlanner {
     fn results<'a>(
         &'a self,
         aggregators: &'a [CompositeAggregator],
-        context: &'a EvaluationContext,
+        headers_index: &'a HeadersIndex,
     ) -> impl Iterator<Item = Result<DynamicValue, SpecifiedEvaluationError>> + 'a {
         self.output_plan.iter().map(move |unit| {
             aggregators[unit.expr_index].get_final_value(
                 unit.aggregator_index,
                 &unit.agg_method,
-                context,
+                headers_index,
             )
         })
     }
@@ -1041,21 +1113,34 @@ impl ConcreteAggregationPlanner {
 // NOTE: parallelizing "horizontally" the planner's execution units does not
 // seem to yield any performance increase. I guess the overhead is greater than
 // the inner computation time.
-fn run_with_record_on_aggregators(
+fn run_with_record_on_aggregators<'a>(
     planner: &ConcreteAggregationPlanner,
-    aggregators: &mut Vec<CompositeAggregator>,
+    aggregators: impl Iterator<Item = &'a mut CompositeAggregator>,
     index: usize,
     record: &ByteRecord,
-    context: &EvaluationContext,
+    headers_index: &HeadersIndex,
+    last_value: Option<DynamicValue>,
 ) -> Result<(), SpecifiedEvaluationError> {
     for (unit, aggregator) in planner.execution_plan.iter().zip(aggregators) {
         let value = match &unit.expr {
             None => None,
-            Some(expr) => Some(eval_expression(expr, Some(index), record, context)?),
+            Some(expr) => Some(eval_expression_with_optional_last_value(
+                expr,
+                Some(index),
+                record,
+                headers_index,
+                last_value.clone(),
+            )?),
         };
 
         if let Some(pair_expr) = &unit.pair_expr {
-            let second_value = eval_expression(pair_expr, Some(index), record, context)?;
+            let second_value = eval_expression_with_optional_last_value(
+                pair_expr,
+                Some(index),
+                record,
+                headers_index,
+                last_value,
+            )?;
 
             return aggregator
                 .process_pair(index, value.unwrap(), second_value)
@@ -1081,23 +1166,32 @@ fn run_with_record_on_aggregators(
 #[derive(Debug, Clone)]
 pub struct AggregationProgram {
     aggregators: Vec<CompositeAggregator>,
+    len: usize,
     planner: ConcreteAggregationPlanner,
-    context: EvaluationContext,
+    headers_index: HeadersIndex,
+    last_value: DynamicValue,
 }
 
 impl AggregationProgram {
     pub fn parse(code: &str, headers: &ByteRecord) -> Result<Self, ConcretizationError> {
         let concrete_aggregations = prepare(code, headers)?;
+        let len = concrete_aggregations.len();
         let planner = ConcreteAggregationPlanner::from(concrete_aggregations);
         let aggregators = planner.instantiate_aggregators();
 
         Ok(Self {
             planner,
             aggregators,
-            context: EvaluationContext::new(headers),
+            len,
+            headers_index: HeadersIndex::from_headers(headers),
+            last_value: DynamicValue::empty_bytes(),
         })
     }
 
+    pub fn has_single_expr(&self) -> bool {
+        self.len == 1
+    }
+
     pub fn clear(&mut self) {
         for aggregator in self.aggregators.iter_mut() {
             aggregator.clear()
@@ -1119,10 +1213,29 @@ impl AggregationProgram {
     ) -> Result<(), SpecifiedEvaluationError> {
         run_with_record_on_aggregators(
             &self.planner,
-            &mut self.aggregators,
+            self.aggregators.iter_mut(),
             index,
             record,
-            &self.context,
+            &self.headers_index,
+            None,
+        )
+    }
+
+    pub fn run_with_cell(
+        &mut self,
+        index: usize,
+        record: &ByteRecord,
+        cell: &[u8],
+    ) -> Result<(), SpecifiedEvaluationError> {
+        self.last_value.set_bytes(cell);
+
+        run_with_record_on_aggregators(
+            &self.planner,
+            self.aggregators.iter_mut(),
+            index,
+            record,
+            &self.headers_index,
+            Some(self.last_value.clone()),
         )
     }
 
@@ -1137,7 +1250,7 @@ impl AggregationProgram {
 
         let mut record = ByteRecord::new();
 
-        for value in self.planner.results(&self.aggregators, &self.context) {
+        for value in self.planner.results(&self.aggregators, &self.headers_index) {
             record.push_field(&value?.serialize_as_bytes());
         }
 
@@ -1151,7 +1264,7 @@ type GroupKey = Vec<Vec<u8>>;
 pub struct GroupAggregationProgram {
     planner: ConcreteAggregationPlanner,
     groups: ClusteredInsertHashmap<GroupKey, Vec<CompositeAggregator>>,
-    context: EvaluationContext,
+    headers_index: HeadersIndex,
 }
 
 impl GroupAggregationProgram {
@@ -1162,7 +1275,7 @@ impl GroupAggregationProgram {
         Ok(Self {
             planner,
             groups: ClusteredInsertHashmap::new(),
-            context: EvaluationContext::new(headers),
+            headers_index: HeadersIndex::from_headers(headers),
         })
     }
 
@@ -1194,7 +1307,14 @@ impl GroupAggregationProgram {
             .groups
             .insert_with(group, || planner.instantiate_aggregators());
 
-        run_with_record_on_aggregators(&self.planner, aggregators, index, record, &self.context)
+        run_with_record_on_aggregators(
+            &self.planner,
+            aggregators.iter_mut(),
+            index,
+            record,
+            &self.headers_index,
+            None,
+        )
     }
 
     pub fn headers(&self) -> impl Iterator<Item = &[u8]> {
@@ -1206,7 +1326,7 @@ impl GroupAggregationProgram {
         parallel: bool,
     ) -> impl Iterator<Item = Result<(GroupKey, ByteRecord), SpecifiedEvaluationError>> {
         let planner = self.planner;
-        let context = self.context;
+        let headers_index = self.headers_index;
 
         self.groups
             .into_iter()
@@ -1217,7 +1337,7 @@ impl GroupAggregationProgram {
 
                 let mut record = ByteRecord::new();
 
-                for value in planner.results(&aggregators, &context) {
+                for value in planner.results(&aggregators, &headers_index) {
                     record.push_field(&value?.serialize_as_bytes());
                 }
 
@@ -1225,3 +1345,221 @@ impl GroupAggregationProgram {
             })
     }
 }
+
+#[derive(Debug, Clone, Default)]
+struct PivotedColumnNamesIndex {
+    names: Vec<Vec<u8>>,
+    seen: BTreeSet<Vec<u8>>,
+}
+
+impl PivotedColumnNamesIndex {
+    fn add(&mut self, name: &[u8]) {
+        if !self.seen.contains(name) {
+            self.seen.insert(name.to_vec());
+            self.names.push(name.to_vec());
+        }
+    }
+}
+
+#[derive(Debug, Clone)]
+pub struct PivotAggregationProgram {
+    planner: ConcreteAggregationPlanner,
+    groups: IndexMap<GroupKey, BTreeMap<Vec<u8>, CompositeAggregator>, RandomState>,
+    headers_index: HeadersIndex,
+    pivoted_column_names_index: PivotedColumnNamesIndex,
+}
+
+impl PivotAggregationProgram {
+    pub fn parse(code: &str, headers: &ByteRecord) -> Result<Self, ConcretizationError> {
+        let concrete_aggregations = prepare(code, headers)?;
+
+        if concrete_aggregations.len() != 1 {
+            return Err(ConcretizationError::Custom(format!(
+                "expected a single aggregation clause, but got {}",
+                concrete_aggregations.len()
+            )));
+        }
+
+        let planner = ConcreteAggregationPlanner::from(concrete_aggregations);
+
+        Ok(Self {
+            planner,
+            groups: IndexMap::with_hasher(RandomState::new()),
+            headers_index: HeadersIndex::from_headers(headers),
+            pivoted_column_names_index: PivotedColumnNamesIndex::default(),
+        })
+    }
+
+    pub fn used_column_indices(&self) -> Vec<usize> {
+        let mut indices = Vec::new();
+
+        for unit in self.planner.execution_plan.iter() {
+            unit.used_column_indices(&mut indices);
+        }
+
+        indices.sort();
+        indices.dedup();
+
+        indices
+    }
+
+    pub fn run_with_record(
+        &mut self,
+        group: GroupKey,
+        pivot: Vec<u8>,
+        index: usize,
+        record: &ByteRecord,
+    ) -> Result<(), SpecifiedEvaluationError> {
+        self.pivoted_column_names_index.add(&pivot);
+
+        let planner = &self.planner;
+
+        let pivot_map = self.groups.entry(group).or_default();
+
+        let aggregator = pivot_map
+            .entry(pivot)
+            .or_insert_with(|| planner.instantiate_single_aggregator());
+
+        run_with_record_on_aggregators(
+            planner,
+            once(aggregator),
+            index,
+            record,
+            &self.headers_index,
+            None,
+        )
+    }
+
+    pub fn pivoted_column_names(&self) -> &[Vec<u8>] {
+        &self.pivoted_column_names_index.names
+    }
+
+    pub fn flush<F, E>(self, mut callback: F) -> Result<(), E>
+    where
+        F: FnMut(&csv::ByteRecord) -> Result<(), E>,
+        E: From<SpecifiedEvaluationError>,
+    {
+        let mut record = csv::ByteRecord::new();
+        let names = self.pivoted_column_names_index.names;
+
+        for (group, mut pivot_map) in self.groups.into_iter() {
+            record.clear();
+
+            for cell in group {
+                record.push_field(&cell);
+            }
+
+            for name in names.iter() {
+                if let Some(aggregator) = pivot_map.get_mut(name) {
+                    aggregator.finalize(false);
+
+                    for value in self
+                        .planner
+                        .results(std::slice::from_ref(aggregator), &self.headers_index)
+                    {
+                        record.push_field(&value?.serialize_as_bytes());
+                    }
+                } else {
+                    record.push_field(b"");
+                }
+            }
+
+            callback(&record)?;
+        }
+
+        Ok(())
+    }
+}
+
+#[derive(Debug, Clone)]
+pub struct GroupAlongColumnsAggregationProgram {
+    planner: ConcreteAggregationPlanner,
+    cols: usize,
+    groups: ClusteredInsertHashmap<GroupKey, Vec<CompositeAggregator>>,
+    headers_index: HeadersIndex,
+    last_value: DynamicValue,
+}
+
+impl GroupAlongColumnsAggregationProgram {
+    pub fn parse(
+        code: &str,
+        headers: &ByteRecord,
+        cols: usize,
+    ) -> Result<Self, ConcretizationError> {
+        let concrete_aggregations = prepare(code, headers)?;
+
+        if concrete_aggregations.len() != 1 {
+            return Err(ConcretizationError::Custom(format!(
+                "expected a single aggregation clause, but got {}",
+                concrete_aggregations.len()
+            )));
+        }
+
+        let planner = ConcreteAggregationPlanner::from(concrete_aggregations);
+
+        Ok(Self {
+            planner,
+            cols,
+            groups: ClusteredInsertHashmap::new(),
+            headers_index: HeadersIndex::from_headers(headers),
+            last_value: DynamicValue::empty_bytes(),
+        })
+    }
+
+    pub fn run_with_cells<'a>(
+        &mut self,
+        group: GroupKey,
+        index: usize,
+        record: &ByteRecord,
+        cells: impl Iterator<Item = &'a [u8]>,
+    ) -> Result<(), SpecifiedEvaluationError> {
+        let planner = &self.planner;
+
+        let aggregators_per_cell = self.groups.insert_with(group, || {
+            (0..self.cols)
+                .map(|_| planner.instantiate_single_aggregator())
+                .collect()
+        });
+
+        for (cell, aggregator) in cells.zip(aggregators_per_cell) {
+            self.last_value.set_bytes(cell);
+
+            run_with_record_on_aggregators(
+                &self.planner,
+                once(aggregator),
+                index,
+                record,
+                &self.headers_index,
+                Some(self.last_value.clone()),
+            )?;
+        }
+
+        Ok(())
+    }
+
+    pub fn into_byte_records(
+        self,
+        parallel: bool,
+    ) -> impl Iterator<Item = Result<(GroupKey, ByteRecord), SpecifiedEvaluationError>> {
+        let planner = self.planner;
+        let headers_index = self.headers_index;
+
+        self.groups
+            .into_iter()
+            .map(move |(group, mut aggregator_per_cell)| {
+                for aggregator in aggregator_per_cell.iter_mut() {
+                    aggregator.finalize(parallel);
+                }
+
+                let mut record = ByteRecord::new();
+
+                for aggregator in aggregator_per_cell.iter() {
+                    for value in planner.results(std::slice::from_ref(aggregator), &headers_index) {
+                        record.push_field(&value?.serialize_as_bytes());
+                    }
+                }
+
+                Ok((group, record))
+            })
+    }
+}
