@@ -1,4 +1,3 @@
-use std::collections::HashMap;
 use std::convert::TryFrom;
 use std::iter;
 use std::sync::Arc;
@@ -9,10 +8,12 @@ use lazy_static::lazy_static;
 use regex::{Captures, Regex};
 use scraper::{Element, ElementRef, Html, Node, Selector};
 
+use crate::collections::HashMap;
+
 use super::error::{ConcretizationError, SpecifiedEvaluationError};
 use super::interpreter::{concretize_expression, ConcreteExpr, EvaluationContext, GlobalVariables};
 use super::parser::{parse_scraper, Expr, ScrapingBrackets, ScrapingNode};
-use super::types::{Argument, DynamicValue, FunctionArguments};
+use super::types::{Argument, DynamicValue, FunctionArguments, HeadersIndex};
 
 trait HtmlExt {
     fn get_element(&self, id: NodeId) -> ElementRef;
@@ -448,8 +449,6 @@ struct ConcreteScrapingLeaf {
 impl ConcreteScrapingLeaf {
     fn evaluate(
         &self,
-        index: Option<usize>,
-        record: &ByteRecord,
         context: &EvaluationContext,
         html: &Html,
         selection: &Selection,
@@ -470,7 +469,7 @@ impl ConcreteScrapingLeaf {
                 let mut globals = GlobalVariables::of("value");
                 globals.set_value(0, value);
 
-                expr.evaluate(index, record, context, Some(&globals), None)
+                expr.evaluate(&context.with_globals(&globals))
             }
         }
     }
@@ -495,18 +494,16 @@ impl ConcreteScrapingNode {
     fn evaluate(
         &self,
         scratch: &mut Vec<DynamicValue>,
-        index: Option<usize>,
-        record: &ByteRecord,
         context: &EvaluationContext,
         html: &Html,
         selection: &Selection,
     ) -> Result<(), SpecifiedEvaluationError> {
         match self {
             Self::Leaf(leaf) => {
-                scratch.push(leaf.evaluate(index, record, context, html, selection)?);
+                scratch.push(leaf.evaluate(context, html, selection)?);
             }
             Self::Brackets(brackets) => {
-                brackets.evaluate(scratch, index, record, context, html, selection)?;
+                brackets.evaluate(scratch, context, html, selection)?;
             }
         };
 
@@ -528,8 +525,6 @@ impl ConcreteScrapingBrackets {
     fn evaluate(
         &self,
         scratch: &mut Vec<DynamicValue>,
-        index: Option<usize>,
-        record: &ByteRecord,
         context: &EvaluationContext,
         html: &Html,
         selection: &Selection,
@@ -537,7 +532,7 @@ impl ConcreteScrapingBrackets {
         let selection = self.selection_expr.evaluate(html, selection);
 
         for node in self.nodes.iter() {
-            node.evaluate(scratch, index, record, context, html, &selection)?;
+            node.evaluate(scratch, context, html, &selection)?;
         }
 
         Ok(())
@@ -555,14 +550,12 @@ impl ConcreteScraper {
     fn evaluate(
         &self,
         scratch: &mut Vec<DynamicValue>,
-        index: Option<usize>,
-        record: &ByteRecord,
         context: &EvaluationContext,
         html: &Html,
         selection: &Selection,
     ) -> Result<(), SpecifiedEvaluationError> {
         for brackets in self.0.iter() {
-            brackets.evaluate(scratch, index, record, context, html, selection)?;
+            brackets.evaluate(scratch, context, html, selection)?;
         }
 
         Ok(())
@@ -769,14 +762,13 @@ fn concretize_scraper(
 #[derive(Debug, Clone)]
 pub struct ScrapingProgram {
     scraper: ConcreteScraper,
-    context: EvaluationContext,
+    headers_index: HeadersIndex,
     pub capacity: usize,
 }
 
 impl ScrapingProgram {
     pub fn parse(code: &str, headers: &ByteRecord) -> Result<Self, ConcretizationError> {
-        let scraper =
-            parse_scraper(code).map_err(|_| ConcretizationError::ParseError(code.to_string()))?;
+        let scraper = parse_scraper(code).map_err(ConcretizationError::ParseError)?;
 
         let concrete_scraper =
             concretize_scraper(scraper, headers, Some(&GlobalVariables::of("value")))?;
@@ -784,7 +776,7 @@ impl ScrapingProgram {
         let capacity = concrete_scraper.names().count();
 
         Ok(Self {
-            context: EvaluationContext::new(headers),
+            headers_index: HeadersIndex::from_headers(headers),
             scraper: concrete_scraper,
             capacity,
         })
@@ -803,14 +795,17 @@ impl ScrapingProgram {
         let selection = Selection::Singular(html.root_element().id());
         let mut scratch = Vec::with_capacity(self.capacity);
 
-        self.scraper.evaluate(
-            &mut scratch,
-            Some(index),
+        let context = EvaluationContext {
+            index: Some(index),
             record,
-            &self.context,
-            html,
-            &selection,
-        )?;
+            headers_index: &self.headers_index,
+            globals: None,
+            lambda_variables: None,
+            last_value: None,
+        };
+
+        self.scraper
+            .evaluate(&mut scratch, &context, html, &selection)?;
 
         Ok(scratch)
     }
@@ -826,14 +821,10 @@ impl ScrapingProgram {
             let selection = Selection::Singular(element.id());
             let mut scratch = Vec::with_capacity(self.capacity);
 
-            self.scraper.evaluate(
-                &mut scratch,
-                Some(index),
-                record,
-                &self.context,
-                html,
-                &selection,
-            )?;
+            let context = EvaluationContext::new(Some(index), record, &self.headers_index);
+
+            self.scraper
+                .evaluate(&mut scratch, &context, html, &selection)?;
 
             Ok(scratch)
         })
