@@ -1,5 +1,5 @@
 use std::borrow::Cow;
-use std::collections::{HashMap, VecDeque};
+use std::collections::VecDeque;
 use std::fmt;
 
 use std::sync::Arc;
@@ -14,6 +14,7 @@ use serde::{
 };
 use url::Url;
 
+use crate::collections::HashMap;
 use crate::dates;
 use crate::moonblade::error::EvaluationError;
 use crate::moonblade::utils::downgrade_float;
@@ -182,6 +183,10 @@ impl DynamicValue {
         Self::Bytes(Arc::new(BString::from(bytes)))
     }
 
+    pub fn empty_bytes() -> Self {
+        Self::from_owned_bytes(b"".to_vec())
+    }
+
     pub fn type_of(&self) -> &str {
         match self {
             Self::List(_) => "list",
@@ -459,6 +464,23 @@ impl DynamicValue {
     pub fn flat_iter(&self) -> DynamicValueFlatIter {
         DynamicValueFlatIter::new(self)
     }
+
+    pub fn set_bytes(&mut self, new_bytes: &[u8]) {
+        match self {
+            Self::Bytes(bytes) => {
+                // NOTE: I cannot really prove this is faster to avoid allocation here...
+                // It certainly seems a little bit faster but not by a large margin.
+                match Arc::get_mut(bytes) {
+                    Some(inner) => {
+                        inner.clear();
+                        inner.extend(new_bytes);
+                    }
+                    None => *bytes = Arc::new(BString::new(new_bytes.to_vec())),
+                };
+            }
+            _ => panic!("DynamicValue is not Bytes!"),
+        }
+    }
 }
 
 pub struct DynamicValueFlatIter<'a> {
