@@ -49,7 +49,8 @@ pub fn version() -> String {
 
 lazy_static! {
     static ref FLAG_REGEX: Regex = Regex::new(r"([\s,/\(])(--?[A-Za-z§][\w\-]*)").unwrap();
-    static ref SECTION_REGEX: Regex = Regex::new("(?im)^.*(?:usage|options?):|---+").unwrap();
+    static ref SECTION_REGEX: Regex =
+        Regex::new("(?im)^.*(?:usage|options|commands?):|---+").unwrap();
     static ref DIMMED_REGEX: Regex = Regex::new(
         r"\[--\]|\[?<[\w|\-]+>(?:\.{3})?\]?|\[[\w\s:§|\-.]+\]|\s+[\$>][^\n]+|\*[^*\n]+\*"
     )
